/**
 *  This file is part of MetaModel.
 *
 *  MetaModel is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  MetaModel is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with MetaModel.  If not, see <http://www.gnu.org/licenses/>.
 */
package dk.eobjects.metamodel.detect;

import java.util.Arrays;
import java.util.Map;

import dk.eobjects.metamodel.data.AbstractDataSet;
import dk.eobjects.metamodel.data.DataSet;
import dk.eobjects.metamodel.data.DefaultRow;
import dk.eobjects.metamodel.data.Row;
import dk.eobjects.metamodel.query.SelectItem;
import dk.eobjects.metamodel.schema.Column;

public class ColumnTypeTransformerDataSet extends AbstractDataSet {

	private DataSet _dataSet;
	private Map<String, ColumnTypeTransformer> _columnTypeTransformers;
	private SelectItem[] _selectItems;

	public ColumnTypeTransformerDataSet(DataSet dataSet,
			Map<String, ColumnTypeTransformer> columnTypeTransformers) {
		_dataSet = dataSet;
		_selectItems = dataSet.getSelectItems();
		_columnTypeTransformers = columnTypeTransformers;
	}

	public void close() {
		_dataSet.close();
		_dataSet = null;
		_columnTypeTransformers = null;
	}

	@Override
	public Row getRow() {
		Object[] transformedValues = new Object[_selectItems.length];
		Row row = _dataSet.getRow();
		for (int i = 0; i < _selectItems.length; i++) {
			SelectItem selectItem = _selectItems[i];
			Column column = selectItem.getColumn();
			ColumnTypeTransformer columnTypeTransformer = _columnTypeTransformers
					.get(column.getQualifiedLabel());
			Object untransformedValue = row.getValue(i);
			if (columnTypeTransformer != null
					&& untransformedValue instanceof String) {
				transformedValues[i] = columnTypeTransformer
						.narrowValue((String) untransformedValue);
			} else {
				transformedValues[i] = untransformedValue;
			}
		}
		return new DefaultRow(_selectItems, transformedValues);
	}

	@Override
	public SelectItem[] getSelectItems() {
		return Arrays.copyOf(_selectItems, _selectItems.length);
	}

	@Override
	public boolean next() {
		if (_dataSet != null) {
			return _dataSet.next();
		}
		return false;
	}

}
